# SPDX-FileCopyrightText: 2019 Bryan Siepert for Adafruit Industries
#
# SPDX-License-Identifier: MIT

from time import sleep
import board
import busio
import adafruit_lsm303_accel
import adafruit_lis2mdl
import os
import math

i2c = busio.I2C(board.SCL, board.SDA)
mag = adafruit_lis2mdl.LIS2MDL(i2c)
accel = adafruit_lsm303_accel.LSM303_Accel(i2c)

x=0
### All values in MicroTeslas

## Initial hard iron compensation
fp = open("hard_iron_offsets","r");
offsets = fp.read();
fp.close()

print ("Initial offsets: " + offsets + "\n");

offset_a = offsets.split();
x_corr = int(offset_a[0])
#print ("X:" + mag_x)
y_corr = int(offset_a[1])
#print ("Y:" + mag_y)
z_corr = int(offset_a[2])
#print ("Z:" + mag_z)

while True:
    mag_x, mag_y, mag_z = mag.magnetic;
    acc_x, acc_y, acc_z = accel.acceleration;

    mag_x = mag_x - x_corr;
    mag_y = mag_y - y_corr;
    mag_z = mag_z - z_corr;

    sleep(1)
    os.system('clear')

    print ("Absolute calvalues: X=%d Y=%d Z=%d\n"%(x_corr, y_corr, z_corr))
    print("Acceleration (m/s^2): X=%0.3f Y=%0.3f Z=%0.3f"%(acc_x,acc_y, acc_z))
    print("\nMagnetometer     : X=%0.3f Y=%0.3f Z=%0.3f"%mag.magnetic)
    print("\nCalibrated Mag   : X=%0.3f Y=%0.3f Z=%0.3f"%(mag_x, mag_y, mag_z))
    
    
    heading =  ( math.atan2(mag_y,mag_x) ) * 180 / 3.1415
    if (heading < 0) :heading = heading + 360
    print ("uncorrected heading: %0.3f , note this is wrong if tilt != 0 "% (heading));

    #Compensate for tilt here using x, y, z acceleration values
    #pitch = atan (x / sqrt(y^2 + z^2));
    #roll = atan (y / sqrt(z^2 + z^2));
    pitch = - math.atan(acc_x / math.sqrt(acc_y * acc_y + acc_z * acc_z));
    roll = math.atan(acc_y / math.sqrt(acc_z * acc_z + acc_z * acc_z))

    #Convert pitch and roll to degrees
    #Actually, don't.  trig functions below expect radians.
    #pitch = pitch *  180 / 3.1415
    #roll = roll * 180 / 3.1415

    print ("Pitch: %0.3f Roll %0.3f"% (pitch * 180 / 3.1415, roll * 180 / 3.1415))

    xh = mag_x * math.cos(pitch) + mag_y * math.sin(roll) * math.sin(pitch) - mag_z * math.cos(roll) * math.sin(pitch)
    yh = mag_y * math.cos(roll) + mag_z * math.sin(roll)

    tilt_compensated_heading =  ( math.atan2(yh,xh) ) * 180 / 3.1415
    if (tilt_compensated_heading < 0) :tilt_compensated_heading = tilt_compensated_heading + 360

    print ("tilt compensated heading: %0.3f X: "% (tilt_compensated_heading));

    cardinal = "U"
    if ((tilt_compensated_heading > 335) or (tilt_compensated_heading <= 25)) : cardinal = "N";
    if ((tilt_compensated_heading <= 335) and (tilt_compensated_heading > 295)) : cardinal = "NW";
    if ((tilt_compensated_heading <= 295) and (tilt_compensated_heading > 245)) : cardinal = "W";
    if ((tilt_compensated_heading <= 245) and (tilt_compensated_heading > 205)) : cardinal = "SW";
    if ((tilt_compensated_heading <= 205) and (tilt_compensated_heading > 155)) : cardinal = "S";
    if ((tilt_compensated_heading <= 155) and (tilt_compensated_heading > 115)) : cardinal = "SE";
    if ((tilt_compensated_heading <= 115) and (tilt_compensated_heading > 65)) : cardinal = "E";
    if ((tilt_compensated_heading <= 65) and (tilt_compensated_heading > 25)) : cardinal = "NE";

    print ("CARDINAL: " + cardinal);



